import utils
from models.collect_data import CollectData
from models.process_data import ProcessData
from models.process_result_data import ProcessResultData
from stores.data_store_interface import IDataStore


class DataProcess:

    _store = None

    DEFAULT_AI_SYSTEM_PROMPT = "从给定信息中提取出关键信息，并以给定的类型返回json字符串，如果部分信息为空，则该字段返回为空。"
    DEFAULT_AI_PROMPT_TEMPLATE_1 = """在以上内容中提取招标信息：
            ```typescript
            export interface Tender { //招标信息
                no: string; // 招标项目编号
                title: string; // 招标公告标题
                provice: string; // 招标单位省份
                city: string; // 招标单位城市
                date: string; // 项目开标的时间
                address: string; // 项目开标的地点
                release_date: string; // 招标信息的发布时间
                summary: string; // 100字左右的招标条件，联系方式等内容摘要
                devices: string; // 只涉及到光谱仪相关的设备，其他无关设备不需要，多个设备以逗号分割 ，例如 红外光谱仪,拉曼光谱仪等
            }
            ```
            """
    DEFAULT_AI_PROMPT_TEMPLATE_2 = """在以上内容中提取中标信息：
            ```typescript
            export interface Instrument { // 中标仪器信息
              company: string; // 中标单位名称，参与竞标并中标的公司名称
              name: string; // 仪器名称，例如：红外光谱仪
              manufacturer: string; // 仪器厂商，例如：赛默飞、Bruker
              model: string; // 仪器的型号/规格，例如：NIR25S
              quantity: number; // 中标仪器的数量，台数，例如：2
              unit_price: number; // 仪器的单价，单位转换为元，例如：178000.00
            }
            export interface BiddingAcceptance { //中标信息
              no: string; // 项目编号
              title: string; // 中标公告标题
              date: string; // 中标公告时间,格式为yyyy-MM-dd 例如：2025-01-01
              provice: string; // 招标单位省份
              city: string; // 招标单位城市
              summary: string; // 公告摘要信息，100字左右
              instruments: Instrument[]; // 中标设备的信息，关于光谱仪的设备，其他设备不要
            }
            ```
           """

    def __init__(self, store: IDataStore):
        self._store = store
        self._ai_system_prompt = utils.get_config_value(
            "ai.system_prompt", self.DEFAULT_AI_SYSTEM_PROMPT
        )
        self._ai_prompt_template_1 = utils.get_config_value(
            "ai.prompt_template_1", self.DEFAULT_AI_PROMPT_TEMPLATE_1
        )
        self._ai_prompt_template_2 = utils.get_config_value(
            "ai.prompt_template_2", self.DEFAULT_AI_PROMPT_TEMPLATE_2
        )

    @property
    def store(self) -> IDataStore:
        return self._store

    def process(self):
        try:
            urls = self.store.query_urls_to_process()
            for item in urls:
                self._process_item(item)
            self.store.save_process_data(True)
            self.store.save_process_result_data(True)
        except Exception as e:
            utils.get_logger().error(f"数据处理发生异常: {e}")
            raise Exception(f"数据处理发生异常: {e}")

    def _process_item(self, url: str) -> None:
        try:
            utils.get_logger().info(f"START ==>URL：{url}")
            item = self.store.query_one_collect_by_url(url)
            if not item:
                utils.get_logger().info(f"END==> NOT FOUND URL：{url}")
                return
            if item.status == 1:
                utils.get_logger().info(f"ALREADY1 URL：{url}")
                return
            data = (
                self.store.query_one_process_by_url(url)
                if item.data_type == 0
                else self.store.query_one_process_result_by_url(url)
            )
            if data:
                utils.get_logger().info(f"ALREADY2 [{item.data_type}] URL==> {url}")
                return
            data = (
                self._ai_process_1(item)
                if item.data_type == 0
                else self._ai_process_2(item)
            )
            if data:
                old = None
                if data.no:
                    old = (
                        self.store.query_one_process_result_by_no(data.no)
                        if item.data_type == 0
                        else self.store.query_one_process_by_no(data.no)
                    )
                if not old:
                    data.url = url
                    data.keyword = item.keyword
                    data.attach_path = item.attach_path
                    if item.data_type == 0:
                        self.store.insert_process_data(data)
                    else:
                        self.store.insert_process_result_data(data)
                else:
                    if old.url != url:
                        if old.other_urls:
                            old.other_urls += f",{url}"
                        else:
                            old.other_urls = url
                        if item.data_type == 0:
                            self.store.set_process_other_urls(data.url, old.other_urls)
                        else:
                            self.store.set_process_result_other_urls(
                                data.url, old.other_urls
                            )
                    self.store.set_collect_process(old.url)
                    utils.get_logger().info(
                        f"ALREADY 编号: {data.no} URL:{old.other_urls}"
                    )

            utils.get_logger().info("END   ==>" + url)
        except Exception as e:
            utils.get_logger().error(f"数据处理发生异常: {url} {e}")

    def _ai_process_1(self, item: CollectData) -> ProcessData | None:
        try:
            data = utils.call_openai(
                self._ai_system_prompt, f"{item.content} {self._ai_prompt_template_1}"
            )
            # area_str = data.get("area")
            #
            # if "省" in area_str:
            #     area_str_arr = area_str.split("省")
            #     area_str = area_str_arr[1] if len(area_str_arr) > 1 else area_str_arr[0]
            # if "市" in area_str:
            #     area_str_arr = area_str.split("市")
            #     area_str = area_str_arr[0]

            return ProcessData(
                no=data.get("no"),
                title=data.get("title"),
                date=data.get("date"),
                provice=data.get("provice"),
                city=data.get("city"),
                address=data.get("address"),
                devices=data.get("devices"),
                summary=data.get("summary"),
                release_date=data.get("release_date"),
                prompt_tokens=data.get("prompt_tokens"),
                completion_tokens=data.get("completion_tokens"),
                total_tokens=data.get("total_tokens"),
            )
        except Exception as e:
            utils.get_logger().error(f"AI 提取数据失败1: {item.url} {e}")
            return None

    def _ai_process_2(self, item: CollectData) -> ProcessResultData | None:
        try:
            data = utils.call_openai(
                self._ai_system_prompt, f"{item.content} {self._ai_prompt_template_2}"
            )
            result = ProcessResultData(
                no=data.get("no"),
                title=data.get("title"),
                provice=data.get("provice"),
                city=data.get("city"),
                date=data.get("date"),
                instruments_o=data.get("instruments"),
                summary=data.get("summary"),
                prompt_tokens=data.get("prompt_tokens"),
                completion_tokens=data.get("completion_tokens"),
                total_tokens=data.get("total_tokens"),
            )
            return result
        except Exception as e:
            utils.get_logger().error(f"AI 提取数据失败2: {item.url} {e}")
            return None
